<?php
function crop_upload_process_image($image_data) {
    if (!crop_upload_should_process($image_data)) {
        return $image_data;
    }

    $options = crop_upload_get_options();
    
    // Load image editor
    $editor = wp_get_image_editor($image_data['file']);
    if (is_wp_error($editor)) {
        crop_upload_log('Failed to load image editor');
        return $image_data;
    }

    // Get current image size
    $size = $editor->get_size();
    
    // Calculate crop dimensions
    $crop_data = crop_upload_calculate_crop(
        $size['width'],
        $size['height'],
        $options['width'],
        $options['height'],
        $options['position']
    );

    // Perform the crop
    $result = $editor->crop(
        $crop_data['x'],
        $crop_data['y'],
        $crop_data['width'],
        $crop_data['height']
    );

    if (is_wp_error($result)) {
        crop_upload_log('Failed to crop image: ' . $result->get_error_message());
        return $image_data;
    }

    // Save the image
    $editor->set_quality($options['quality']);
    $result = $editor->save($image_data['file']);

    if (is_wp_error($result)) {
        crop_upload_log('Failed to save image: ' . $result->get_error_message());
        return $image_data;
    }

    return $image_data;
}

function crop_upload_calculate_crop($orig_w, $orig_h, $target_w, $target_h, $position) {
    $scale_w = $target_w / $orig_w;
    $scale_h = $target_h / $orig_h;
    $scale = max($scale_w, $scale_h);
    
    $crop_w = round($target_w / $scale);
    $crop_h = round($target_h / $scale);
    
    // Calculate x position
    $x = round(($orig_w - $crop_w) / 2);
    
    // Calculate y position based on selected position
    switch ($position) {
        case 'top':
            $y = 0;
            break;
        case 'bottom':
            $y = $orig_h - $crop_h;
            break;
        default: // center
            $y = round(($orig_h - $crop_h) / 2);
    }
    
    return [
        'x' => $x,
        'y' => $y,
        'width' => $crop_w,
        'height' => $crop_h
    ];
}

function crop_upload_should_process($image_data) {
    $options = crop_upload_get_options();
    
    if (!$options['enabled']) {
        return false;
    }

    $valid_types = ['image/jpeg', 'image/png', 'image/gif'];
    if (!in_array($image_data['type'], $valid_types)) {
        return false;
    }

    // Don't process animated GIFs
    if ($image_data['type'] === 'image/gif' && crop_upload_is_animated_gif($image_data['file'])) {
        return false;
    }

    return true;
}

function crop_upload_is_animated_gif($filename) {
    if (!($fh = @fopen($filename, 'rb'))) {
        return false;
    }

    $count = 0;
    while (!feof($fh) && $count < 2) {
        $chunk = fread($fh, 1024 * 100);
        $count += preg_match_all('#\x00\x21\xF9\x04.{4}\x00[\x2C\x21]#s', $chunk, $matches);
    }

    fclose($fh);
    return $count > 1;
}

function crop_upload_log($message) {
    if (defined('CROP_UPLOAD_DEBUG') && CROP_UPLOAD_DEBUG) {
        error_log('[Crop Upload] ' . $message);
    }
}